package banking;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;

/**
 *  A simple banking DatabaseImpl class.
 *
 *@author     <a href="mailto:Sara.Bouchenak@inria.fr">Sara Bouchenak</a>
 *@version    1.0 - 13/01/2004
 */
public class DatabaseImpl
	implements Database {
	
	/**
	 * The database URL.
	 * 
	 */
	private String	url;
	
	/**
	 * The database admin name.
	 * 
	 */
	private String	user;
	
	/**
	 * The database admin password.
	 * 
	 */
	private String	password;
	
	/**
	 * Constructs a DatabaseImpl with a given URL 
	 * and user name/password.
	 * 
	 * @param url		The database URL
	 * @param user		The database user name
	 * @param password	The database user password
	 * @param number	The user account number
	 * @param customer	The customer name
	 * @param	balance	Amount of initial balanace
	 */
	/*******************************/
	/* ADDING TRANSACTIONS BY HAND */
	/*******************************/
	/*
	 * OLD:
	public DatabaseImpl(String url, String user, 
		String password, int number, String customer,
		float balance)
	{
		Connection 	conn;
		Statement	stmt;
		
		try 
		{
			// Load the database driver
			Class.forName("com.mckoi.JDBCDriver");
		
			this.url		= url;
			this.user		= user;
			this.password	= password;
		
			conn 			= getConnection();

			stmt			= conn.createStatement();
			stmt.executeUpdate("INSERT INTO accounts (id, customer, balance) VALUES ("
				+ number + ", " + customer + ", " + balance + ")");
			stmt.close();
		}
		catch (Exception e)
		{
			System.err.println("Exception: " + e);
			e.printStackTrace();
			return;
		}
	}
	 */ 
	 
	 /*******************************/
	 /* ADDING TRANSACTIONS BY HAND */
	 /*******************************/
	 /*
	  * NEW VERSION:
	  */
	public DatabaseImpl(String url, String user, 
		String password, int number, String customer,
		float balance)
	{
		Connection 	conn = null;
		Statement	stmt;
		
		try 
		{
			// Load the database driver
			Class.forName("com.mckoi.JDBCDriver");
		
			this.url		= url;
			this.user		= user;
			this.password	= password;
		
			conn 			= getConnection();

			stmt			= conn.createStatement();
			stmt.executeUpdate("INSERT INTO accounts (id, customer, balance) VALUES ("
				+ number + ", " + customer + ", " + balance + ")");
			stmt.close();
			
			try
			{
				conn.commit();
			}
			catch (Exception ex)
			{
				System.err.println("Exception: " + ex);
				ex.printStackTrace();
				return;
			}
		}
		catch (Exception e)
		{
			try
			{
				conn.rollback();
			}
			catch (Exception ex)
			{
				System.err.println("Exception: " + ex);
				ex.printStackTrace();
				return;
			}
			System.err.println("Exception: " + e);
			e.printStackTrace();
		}
		finally
		{
			try
			{
				conn.close();
			}
			catch (Exception ex)
			{
				System.err.println("Exception: " + ex);
				ex.printStackTrace();
				return;
			}
		}
	}
	
	/**
	 * Opens a database connection.
	 * 
	 * @return			A database connection
	 */
	public Connection getConnection()
	{
		Connection 	conn;
		
		try
		{
			conn = DriverManager.getConnection(url, user, 
				password);
	 
			/*******************************/
			/* ADDING TRANSACTIONS BY HAND */
			/*******************************/
			// Do not commit automatically after each 
			// SQL request
			conn.setAutoCommit(false);
			
			return conn;
		}
		catch (Exception e)
		{
			System.err.println("Exception: " + e);
			e.printStackTrace();
			return null;
		}
	}

	 /**
	  * Queries the database to return the balance of 
	  * an account given its number.
	  * 
	  * @param 	accountNumber	The account number.
	  * @return	The account balance.
	  * @throws	InconsistentDatabaseException if a problem occurs 
	  * when querying the database.
	  */
	 public float query(int accountNumber)
		 throws InconsistentDatabaseException
	 {
		 Connection 			conn;
		 Statement			stmt;
		 ResultSet			rs;
		 ResultSetMetaData	rsmd;
		 int				numCols = 0;
		 float				balance = -1;
		
		 try
		 {
			 conn 	= getConnection();
	 
			/*******************************/
			/* ADDING TRANSACTIONS BY HAND */
			/*******************************/
			// Commit automatically after each 
			// SQL request
			conn.setAutoCommit(true);
			
			 stmt	= conn.createStatement();
			 rs		= stmt.executeQuery("SELECT balance FROM accounts WHERE id=" 
				 + accountNumber);
				
			 rsmd	= rs.getMetaData();
			 numCols	= rsmd.getColumnCount();
		 }
		 catch (Exception e)
		 {
			 throw (new InconsistentDatabaseException(e.getMessage()));
		 }
			
		 if (numCols == 0)
		 {
			 try
			 {
				 rs.close();
			 }
			 catch (Exception e)
			 {
				 throw (new InconsistentDatabaseException(e.getMessage()));
			 }
			
			 throw (new InconsistentDatabaseException("No such account with the number '" 
				 + accountNumber + "'"));
		 }
		 if (numCols > 1)
		 {
			 try
			 {
				 rs.close();
			 }
			 catch (Exception e)
			 {
				 throw (new InconsistentDatabaseException(e.getMessage()));
			 }
			
			 throw (new InconsistentDatabaseException("More than one account with the same number '" 
				 + accountNumber + "'"));
		 }

		 // numCols == 1
		 try
		 {
			 if (rs.next())
				 balance	= rs.getFloat("balance");
				
			 rs.close();
			 stmt.close();
		 }
		 catch (Exception e)
		 {
			 throw (new InconsistentDatabaseException(e.getMessage()));
		 }
		
		 return balance;
	 }
	 
	 /*******************************/
	 /* ADDING TRANSACTIONS BY HAND */
	 /*******************************/
	 /*
	  * NEW VERSION:
	  */
	/**
	 * Queries the database to return the balance of 
	 * an account given its number.
	 * 
	 * @param 	accountNumber	The account number.
	 * @param	conn			The database connection.
	 * @return	The account balance.
	 * @throws	InconsistentDatabaseException if a problem occurs 
	 * when querying the database.
	 */
	public float query(int accountNumber, Connection conn)
		throws InconsistentDatabaseException
	{
		Statement			stmt;
		ResultSet			rs;
		ResultSetMetaData	rsmd;
		int					numCols = 0;
		float				balance = -1;
		
		try
		{
			stmt	= conn.createStatement();
			rs		= stmt.executeQuery("SELECT balance FROM accounts WHERE id=" 
				+ accountNumber);
				
			rsmd	= rs.getMetaData();
			numCols	= rsmd.getColumnCount();
		}
		catch (Exception e)
		{
			throw (new InconsistentDatabaseException(e.getMessage()));
		}
			
		if (numCols == 0)
		{
			try
			{
				rs.close();
			}
			catch (Exception e)
			{
				throw (new InconsistentDatabaseException(e.getMessage()));
			}
			
			throw (new InconsistentDatabaseException("No such account with the number '" 
				+ accountNumber + "'"));
		}
		if (numCols > 1)
		{
			try
			{
				rs.close();
			}
			catch (Exception e)
			{
				throw (new InconsistentDatabaseException(e.getMessage()));
			}
			
			throw (new InconsistentDatabaseException("More than one account with the same number '" 
				+ accountNumber + "'"));
		}

		// numCols == 1
		try
		{
			if (rs.next())
				balance	= rs.getFloat("balance");
				
			rs.close();
			stmt.close();
		}
		catch (Exception e)
		{
			throw (new InconsistentDatabaseException(e.getMessage()));
		}
		return balance;
	}
	
	/**
	 * Updates the database to set the new amount of 
	 * an account balance given its number.
	 * 
	 * @param accountNumber	The account number.
	 * @param amount		The balance to be set
	 * @throws	InconsistentDatabaseException if a problem occurs 
	 * when updating the database.
	 */
	public void update(int accountNumber, float amount)
		throws InconsistentDatabaseException
	{
		Connection 	conn;
		Statement	stmt;
		
		try
		{
			conn 	= getConnection();
	 
			/*******************************/
			/* ADDING TRANSACTIONS BY HAND */
			/*******************************/
			// Commit automatically after each 
			// SQL request
			conn.setAutoCommit(true);
			
			stmt	= conn.createStatement();
			stmt.executeUpdate("UPDATE accounts SET balance=" 
				+ amount + " WHERE id=" + accountNumber);
			stmt.close();
		}
		catch (Exception e)
		{
			throw (new InconsistentDatabaseException(e.getMessage()));
		}
	}
	 
	/*******************************/
	/* ADDING TRANSACTIONS BY HAND */
	/*******************************/
	/*
	 * NEW VERSION:
	 */
	 public void update(int accountNumber, float amount, 
	 	Connection conn)
		 throws InconsistentDatabaseException
	 {
		 Statement	stmt;
		
		 try
		 {
			 stmt	= conn.createStatement();
			 stmt.executeUpdate("UPDATE accounts SET balance=" 
				 + amount + " WHERE id=" + accountNumber);
			 stmt.close();
		 }
		 catch (Exception e)
		 {
			 throw (new InconsistentDatabaseException(e.getMessage()));
		 }
	 }
	
}