package banking;

/*******************************/
/* ADDING TRANSACTIONS BY HAND */
/*******************************/
import java.sql.Connection;

/**
 *  A class that provides inter-account operations.
 *
 *@author     <a href="mailto:Sara.Bouchenak@inria.fr">Sara Bouchenak</a>
 *@version    1.0 - 13/01/2004
 */
public class InterAccountOperations {
		
	/**
	 * Performs funds transfer between two banking accounts.
	 * 
	 * @param	from	The origin account for funds transfer
	 * @param	to		The target account for funds transfer
	 * @param	amount	The amount to transfer
	 * @throws	AccountException	If a problem occurs during
	 * the <code>debit</code> operation, e.g., insufficient balance.
	 */
	/*******************************/
	/* ADDING TRANSACTIONS BY HAND */
	/*******************************/
	/*
	 * OLD:
	public static void transfer(Account from, Account to, 
		float amount)
		throws AccountException
	{
		// Adds the amount to the 'to' account
		to.credit(amount);
		// Deducts the amount from the 'from' account
		from.debit(amount);
	}
	*/
	public static void transfer(Account from, Account to, 
		float amount)
		throws AccountException
	{
		Database 	db = ((DatabaseAccountImpl) from).getDatabase();
		Connection	conn = db.getConnection();
		
		try
		{
			//conn.setAutoCommit(false);
			// Adds the amount to the 'to' account
			((DatabaseAccountImpl) to).credit(amount, conn);
			// Deducts the amount from the 'from' account
			((DatabaseAccountImpl) from).debit(amount, conn);
			
			try
			{
				conn.commit();
			}
			catch (Exception ex)
			{
				System.err.println("Exception: " + ex);
				ex.printStackTrace();
				return;
			}
		}
		catch (AccountException e)
		{
			try
			{
				conn.rollback();
			}
			catch (Exception ex)
			{
				System.err.println("Exception: " + ex);
				ex.printStackTrace();
				return;
			}
			System.err.println("Exception: " + e);
			e.printStackTrace();
			throw e;
		}
		finally
		{
			try
			{
				conn.close();
			}
			catch (Exception ex)
			{
				System.err.println("Exception: " + ex);
				ex.printStackTrace();
				return;
			}
		}
	}
	
}